
//
// This source file is part of appleseed.
// Visit http://appleseedhq.net/ for additional information and resources.
//
// This software is released under the MIT license.
//
// Copyright (c) 2010-2013 Francois Beaune, Jupiter Jazz Limited
// Copyright (c) 2014-2017 Francois Beaune, The appleseedhq Organization
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifndef APPLESEED_RENDERER_KERNEL_RENDERING_IPASSCALLBACK_H
#define APPLESEED_RENDERER_KERNEL_RENDERING_IPASSCALLBACK_H

// appleseed.foundation headers.
#include "foundation/core/concepts/iunknown.h"

// appleseed.main headers.
#include "main/dllsymbol.h"

// Forward declarations.
namespace foundation    { class IAbortSwitch; }
namespace foundation    { class JobQueue; }
namespace renderer      { class Frame; }

namespace renderer
{

//
// Pass callback interface.
//
// Used by the generic frame renderer in multi-pass mode.
//

class APPLESEED_DLLSYMBOL IPassCallback
  : public foundation::IUnknown
{
  public:
    // This method is called at the beginning of a pass.
    virtual void on_pass_begin(
        const Frame&                frame,
        foundation::JobQueue&       job_queue,
        foundation::IAbortSwitch&   abort_switch) = 0;

    // This method is called at the end of a pass.
    virtual void on_pass_end(
        const Frame&                frame,
        foundation::JobQueue&       job_queue,
        foundation::IAbortSwitch&   abort_switch) = 0;
};

}       // namespace renderer

#endif  // !APPLESEED_RENDERER_KERNEL_RENDERING_IPASSCALLBACK_H
