
//
// This source file is part of appleseed.
// Visit http://appleseedhq.net/ for additional information and resources.
//
// This software is released under the MIT license.
//
// Copyright (c) 2014-2017 Srinath Ravichandran, The appleseedhq Organization
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifndef APPLESEED_RENDERER_MODELING_OBJECT_CURVEOBJECTREADER_H
#define APPLESEED_RENDERER_MODELING_OBJECT_CURVEOBJECTREADER_H

// appleseed.renderer headers.
#include "renderer/modeling/object/curveobject.h"

// appleseed.foundation headers.
#include "foundation/utility/autoreleaseptr.h"

// appleseed.main headers.
#include "main/dllsymbol.h"

// Forward declarations.
namespace foundation    { class SearchPaths; }
namespace renderer      { class ParamArray; }

namespace renderer
{

//
// Curve object reader.
//

class APPLESEED_DLLSYMBOL CurveObjectReader
{
  public:
    // Read a curve object from disk. The filepath is defined in params.
    static foundation::auto_release_ptr<CurveObject> read(
        const foundation::SearchPaths&  search_paths,
        const char*                     name,
        const ParamArray&               params);

  private:
    static foundation::auto_release_ptr<CurveObject> create_hair_ball(
        const char*                     name,
        const ParamArray&               params);

    static foundation::auto_release_ptr<CurveObject> create_furry_ball(
        const char*                     name,
        const ParamArray&               params);

    static foundation::auto_release_ptr<CurveObject> load_text_curve_file(
        const foundation::SearchPaths&  search_paths,
        const char*                     name,
        const ParamArray&               params);

    static foundation::auto_release_ptr<CurveObject> load_mitsuba_curve_file(
        const foundation::SearchPaths&  search_paths,
        const char*                     name,
        const ParamArray&               params);
};

}       // namespace renderer

#endif  // !APPLESEED_RENDERER_MODELING_OBJECT_CURVEOBJECTREADER_H
