
//
// This source file is part of appleseed.
// Visit http://appleseedhq.net/ for additional information and resources.
//
// This software is released under the MIT license.
//
// Copyright (c) 2015-2017 Francois Beaune, The appleseedhq Organization
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in
// all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
// THE SOFTWARE.
//

#ifndef APPLESEED_RENDERER_UTILITY_AUTODESKMAX_H
#define APPLESEED_RENDERER_UTILITY_AUTODESKMAX_H

// Standard headers.
#include <cmath>

namespace renderer
{

template <typename T>
inline T autodesk_max_decay(
    const T distance,
    const T decay_start,
    const T decay_exponent)
{
    if (distance < decay_start || decay_exponent == T(0.0))
        return T(1.0);

    const T s = decay_start / distance;

    return
        decay_exponent == T(1.0) ? s :
        decay_exponent == T(2.0) ? s * s :
        std::pow(s, decay_exponent);
}

}       // namespace renderer

#endif  // !APPLESEED_RENDERER_UTILITY_AUTODESKMAX_H
